﻿using System;
using System.Collections.Generic;
using System.Linq;
using AzureTags.Core.Domain;
using AzureTags.Core.Interfaces;

namespace AzureTags.Core.Infrastructure
{
    public class TableStorageResourceRepository
    {
        public IEnumerable<UserTaggedResource> ListUserTaggedResources(string userId)
        {
            var ctx = new ResourceDataContext();
            var resources = (from t in ctx.UserTaggedResourceTable
                             where t.PartitionKey == userId 
                             select t).ToList();
            return FromUserTaggedResourceRowList(resources);
        }


        public void SaveUserTaggedResource(UserTaggedResource resource)
        {
            var myUserTaggedResourceRow = new UserTaggedResourceRow(resource);

            var svc = new ResourceDataContext();

            svc.AddObject(ResourceDataContext.UserTaggedResourceTableName, myUserTaggedResourceRow);
            svc.SaveChanges();
        }

        public int TotalTags()
        {
            var ctx = new ResourceDataContext();
            var allResources = (from t in ctx.UserTaggedResourceTable
                             select t).ToArray();
            return allResources.Count();
        }

        #region Mapping Methods
        private UserTaggedResource FromUserTaggedResourceRow(UserTaggedResourceRow resourceRow)
        {
            return new UserTaggedResource(new Resource(resourceRow.Url), resourceRow.Tag, resourceRow.UserId, resourceRow.Title, resourceRow.Description);
        }

        private List<UserTaggedResource> FromUserTaggedResourceRowList(List<UserTaggedResourceRow> resourceRows)
        {
            var myResources = new List<UserTaggedResource>();
            foreach (var row in resourceRows)
            {
                myResources.Add(FromUserTaggedResourceRow(row));
            }
            return myResources;
        }

        #endregion
    }
}




/*

        public List<Resource> ListResources()
        {
            var ctx = new ResourceDataContext();
            List<ResourceRow> resources = (from t in ctx.ResourceTable
                                           select t).ToList();
            return FromResourceRowList(resources);
        }

        public void SaveResource(string url)
        {
            var myResource = new Resource(url);
            var myResourceRow = new ResourceRow(myResource);

            var svc = new ResourceDataContext();

            svc.AddObject(ResourceDataContext.ResourceTableName, myResourceRow);
            svc.SaveChanges();
        }
 
        private Resource FromResourceRow(ResourceRow resourceRow)
        {
            return new Resource(resourceRow.Url);
        }

        private List<Resource> FromResourceRowList(List<ResourceRow> resourceRows)
        {
            var myResources = new List<Resource>();
            foreach (ResourceRow row in resourceRows)
            {
                myResources.Add(FromResourceRow(row));
            }
            return myResources;
        }


*/