﻿using System;
using System.Net.Mail;
using System.Threading;
using Microsoft.Samples.ServiceHosting.StorageClient;
using Microsoft.ServiceHosting.ServiceRuntime;
using System.Net;

namespace CloudService1_WorkerRole
{
    public class WorkerRole : RoleEntryPoint
    {
        public override void Start()
        {
            MessageQueue queue =
                QueueStorage.Create(StorageAccountInfo.GetDefaultQueueStorageAccountFromConfiguration()).GetQueue(
                    "loginqueue");
            //queue.CreateQueue();


            RoleManager.WriteToLog("Information", "Worker Process entry point called");

            while (true)
            {
                Message msg = queue.GetMessage();
                if (msg != null)
                {
                    RoleManager.WriteToLog("Information",
                                           string.Format("Got loginqueue message '{0}'", msg.ContentAsString()));

                    try
                    {
                        string message = msg.ContentAsString();
                        string[] messageParts = message.Split('|');
                        string toEmail = "ssmith.lists@gmail.com";
                        try
                        {
                            toEmail = messageParts[3];
                        }
                        catch (Exception)
                        {
                            // do nothing and use default email
                        }
                        string fromEmail = "noreply@azuretags.com";
                        var myMessage = new MailMessage(fromEmail, toEmail)
                                            {
                                                Subject = "[AzureTags] New Resource Tagged"
                                            };

                        myMessage.Body = String.Format("{0}\n{1}\ntagged with {2}.", messageParts);
                        myMessage.ReplyTo = new MailAddress("noreply@azuretags.com");

                        try
                        {
                            SendViaLocalhost(myMessage);

                        }
                        catch (Exception)
                        {
                            SendViaGmail(myMessage);
                        }
                    }
                    catch (Exception ex)
                    {
                        RoleManager.WriteToLog("Information",
                                               string.Format("Error Sending Email: {0}", ex));
                    }

                    queue.DeleteMessage(msg);
                }
                else
                {
                    Thread.Sleep(TimeSpan.FromSeconds(5));
                }
            }
        }

        private void SendViaLocalhost(MailMessage myMessage)
        {
            var server = new SmtpClient();
            server.Host = "localhost";
            server.Port = 25;

            server.Send(myMessage);
            RoleManager.WriteToLog("Information",
                       string.Format("Email Sent Via Localhost SMTP"));
        }

        private void SendViaGmail(MailMessage myMessage)
        {
            var server = new SmtpClient();
            server.Host = "smtp.gmail.com";
            server.Port = 587;
            GetGmailCredentials(server);
            server.EnableSsl = true;

            server.Send(myMessage);
            RoleManager.WriteToLog("Information",
                       string.Format("Email Sent Via GMail SMTP"));
        }

        #region GetGmailCredentials
        private void GetGmailCredentials(SmtpClient server)
        {
            server.Credentials = new NetworkCredential("ssmith.lists@gmail.com", "sarkkras");
        }
        #endregion

        public override RoleStatus GetHealthStatus()
        {
            MessageQueue queue =
                QueueStorage.Create(StorageAccountInfo.GetDefaultQueueStorageAccountFromConfiguration()).GetQueue(
                    "loginqueue");

            RoleManager.WriteToLog("Information", string.Format("Queue Count: '{0}'", queue.ApproximateCount()));

            return RoleStatus.Healthy;
        }
    }
}