﻿using System;
using System.Web.UI;
using System.Web;
using System.IO;
namespace ASPNETTipsAndTricks.Code
{
    public class BasePage : System.Web.UI.Page
    {
        #region FindControl
        public T FindControl<T>(string id) where T : Control
        {
            return FindControl<T>(Page, id);
        }

        public static T FindControl<T>(Control startingControl, string id) where T : Control
        {
            T found = null;
            foreach (Control activeControl in startingControl.Controls)
            {
                found = activeControl as T;
                if (found == null)
                {
                    found = FindControl<T>(activeControl, id);
                }
                else if (string.Compare(id, found.ID, true) != 0)
                {
                    found = null;
                }
                if (found != null)
                {
                    break;
                }
            }
            return found;
        }
        #endregion

        #region Global Trace Settings
        protected void SetTrace()
        {
#if DEBUG
            // use configured settings
            // this.Trace.IsEnabled = true;
#else
            // ensure tracing is never enabled in production
            this.Trace.IsEnabled = false;
#endif
        }
        #endregion

        protected override void OnPreRender(System.EventArgs e)
        {
            SetTrace();
            base.OnPreRender(e);
        }

        #region PrintView
        public bool PrintView { get; set; }

        override protected void OnInit(EventArgs e)
        {
            base.OnInit(e);
            if (null != Request.QueryString["Print"])
            {
                try { PrintView = Convert.ToBoolean(Request.QueryString["Print"]); }
                catch
                {
                }
            }
        }
        #endregion

        #region Trace Page Size
        protected override void Render(HtmlTextWriter writer)
        {
            if (Trace.IsEnabled)
            {
                System.IO.StringWriter sw = new StringWriter();
                HtmlTextWriter localWriter = new HtmlTextWriter(sw);
                base.Render(localWriter);
                string output = sw.ToString();
                int pageSize = output.Length;
                Trace.Warn("Page Size: " + pageSize.ToString());
                writer.Write(output);
            }
            else
            {
                base.Render(writer);
            }
        }
        #endregion

        #region Trace Viewstate Size
        protected override void SavePageStateToPersistenceMedium(object viewState)
        {
            base.SavePageStateToPersistenceMedium(viewState);
            if (Trace.IsEnabled)
            {
                LosFormatter format = new LosFormatter();
                StringWriter writer = new StringWriter();
                format.Serialize(writer, viewState);
                int viewStateLength = writer.ToString().Length;
                Trace.Warn("ViewState Size: " + viewStateLength.ToString());
            }
        }

        #endregion

        #region RequireSSL
        public bool RequireSSL { get; set; }

        [System.Diagnostics.DebuggerStepThrough]
        public static void RedirectSecureAsNeeded(bool requireSSL)
        {
            const string SECURE = "https://";
            const string UNSECURE = "http://";

            //Force required into secure channel
            if (requireSSL && HttpContext.Current.Request.IsSecureConnection == false)
                HttpContext.Current.Response.Redirect(HttpContext.Current.Request.Url.ToString().Replace(UNSECURE, SECURE));

            //Force non-required out of secure channel
            if (!requireSSL && HttpContext.Current.Request.IsSecureConnection == true)
                HttpContext.Current.Response.Redirect(HttpContext.Current.Request.Url.ToString().Replace(SECURE, UNSECURE));
        }
        #endregion
    }
}
