﻿using System;
using System.ComponentModel;
using System.Configuration;
using System.Data;
using System.Data.SqlClient;
using System.Web;
using System.Web.Caching;
using System.Web.Services;

namespace ASPNETTipsAndTricks
{
    /// <summary>
    /// Summary description for DemoService
    /// </summary>
    [WebService(Namespace = "http://tempuri.org/")]
    [WebServiceBinding(ConformsTo = WsiProfiles.BasicProfile1_1)]
    [ToolboxItem(false)]
    // To allow this Web Service to be called from script, using ASP.NET AJAX, uncomment the following line. 
    [System.Web.Script.Services.ScriptService]
    public class DemoService : System.Web.Services.WebService
    {

        [WebMethod]
        public string HelloWorld()
        {
            return "Hello World";
        }

        [WebMethod]
        public decimal GetTickerPrice(string ticker)
        {
            int rnd = new Random().Next(2000);
            System.Threading.Thread.Sleep(rnd*2);
            return (decimal)rnd / 200;
        }

        [WebMethod]
        [System.Web.Script.Services.ScriptMethod]
        public string GetStats()
        {
            string outputHtml = "<b>Complex Op - Thread ID:</b> ";
            System.Random rnd = new Random();
            int sleeptime = rnd.Next(1000, 2000);
            System.Threading.Thread.Sleep(sleeptime);
            outputHtml += System.Threading.Thread.CurrentThread.ManagedThreadId.ToString() + " : " + sleeptime + "ms";
            return outputHtml;
        }

        [WebMethod]
        [System.Web.Script.Services.ScriptMethod]
        public string NorthwindSales(int productId, string cacheKey)
        {
            string outputHtml = HttpRuntime.Cache[cacheKey] as string;

            if (outputHtml == null)
            {
                try
                {
                    DateTime startDate = DateTime.Today.AddDays(-60);
                    DateTime endDate = DateTime.Today;
                    DataTable myData;

                    myData = GetProductQuantitySold(productId);

                    if (myData.Rows.Count > 0)
                    {
                        outputHtml =
                            ViewManager.RenderView("~/09_Async/TemplateDriven/View_SalesData.ascx", myData);
                    }
                    HttpRuntime.Cache.Add(cacheKey, outputHtml, null, DateTime.Now.AddSeconds(15), TimeSpan.Zero, CacheItemPriority.Normal, null);
                }
                catch (Exception ex)
                {
                    outputHtml = "<p class=\"bodyLarger\" style=\"padding: 5px 14px 10px 14px; text-align: left;\">Something's wrong.</p>";
                    outputHtml = ex.ToString();
                }
            }
            return outputHtml;

        }

        /// <summary>
        /// This would go in a data access layer
        /// </summary>
        /// <param name="productId"></param>
        /// <returns></returns>
        public static DataTable GetProductQuantitySold(int productId)
        {
            string query =
                @"
SELECT Products.ProductID, Products.ProductName, 
COUNT([Order Details].Quantity) AS [Total Quantity Sold]
FROM Products 
INNER JOIN [Order Details] 
ON Products.ProductID = [Order Details].ProductID
WHERE (Products.ProductID = @ProductID)
GROUP BY Products.ProductID, Products.ProductName";

            using (SqlConnection conn =
    new SqlConnection(ConfigurationManager.AppSettings["ConnectionString"].ToString()))
            {
                using (SqlDataAdapter cmd = new SqlDataAdapter(query, conn))
                {
                    cmd.SelectCommand.Parameters.AddWithValue("@ProductID", productId);
                    DataTable dt = new DataTable();
                    cmd.Fill(dt);

                    // Make it slow
                    System.Random rnd = new Random();
                    int sleeptime = rnd.Next(1000, 2000);
                    System.Threading.Thread.Sleep(sleeptime);

                    return dt;
                }
            }
        }

    }
}
